/*
** mod_auth_sha1_ldapstyle.c
**     -- Netscape/LDAP style SHA-1 passwd authentication.
**
**  To play with this module, first compile it into a DSO
**  file and instlal it into Apache 2.0.x's libexec directory
**  by running:
**
**    $ apxs -c -i mod_auth_sha1_ldapstyle.c
**
**  Copyright (C) 2003, Hiroyuki OYAMA. Japan. All rights reserved.
**
*/
#include "apr_sha1.h"

#include "ap_config.h"
#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_protocol.h"
#include "http_request.h"


extern module AP_MODULE_DECLARE_DATA auth_module;
typedef struct {
    char *auth_pwfile;
    char *auth_grpfile;
    int auth_authoritative;
} auth_config_rec;


static char *find_passwd(request_rec *r, char *user, char *auth_pwfile)
{
    ap_configfile_t *f;
    char l[MAX_STRING_LEN];
    const char *rpw, *w;
    apr_status_t status;

    if ((status = ap_pcfg_openfile(&f, r->pool, auth_pwfile)) != APR_SUCCESS) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, status, r,
                      "Could not open password file: %s", auth_pwfile);
        return NULL;
    }
    while (!(ap_cfg_getline(l, MAX_STRING_LEN, f))) {
        if ((l[0] == '#') || (!l[0]))
            continue;
        rpw = l;
        w = ap_getword(r->pool, &rpw, ':');
        if (strcmp(user, w) == 0) {
            ap_cfg_closefile(f);
            return ap_getword(r->pool, &rpw, ':');
        }
    }
    ap_cfg_closefile(f);
    return NULL;
}


static int authenticate_sha1_ldapstyle_basic_user(request_rec *r)
{
    auth_config_rec *conf = ap_get_module_config(r->per_dir_config,
                                                 &auth_module);
    const char *sent_pw;
    char *hash, comp_hash[120];
    int res;

    if ((res = ap_get_basic_auth_pw(r, &sent_pw)))
        return res;
    if (!conf->auth_pwfile)
        return DECLINED;

    if (!(hash = find_passwd(r, r->user, conf->auth_pwfile))) {
        if (!(conf->auth_authoritative))
            return DECLINED;
        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                      "user %s not found(ldapstyle): %s", r->user, r->uri);
        ap_note_basic_auth_failure(r);
        return HTTP_UNAUTHORIZED;
    }
    if (strncmp(hash, APR_SHA1PW_ID, APR_SHA1PW_IDLEN) != 0) {
        return DECLINED;
    }

    /* compute Netscape/LDAP Style SHA1 password string */
    apr_sha1_base64(sent_pw, strlen(sent_pw), comp_hash);
    if (strcmp(comp_hash, hash) != 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                      "user %s: authentication failure for \"%s\"(ldapstyle): "
                      "Password Mismatch",
                      r->user, r->uri);
        ap_note_basic_auth_failure(r);
        return HTTP_UNAUTHORIZED;
    }

    return OK;
}

static void register_hooks(apr_pool_t *p)
{
    static const char * const successor[] = { "mod_auth.c", NULL };

    ap_hook_check_user_id(authenticate_sha1_ldapstyle_basic_user,
                          NULL, successor, APR_HOOK_MIDDLE);
}

/* Dispatch list for API hooks */
module AP_MODULE_DECLARE_DATA auth_sha1_ldapstyle_module = {
    STANDARD20_MODULE_STUFF, 
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    register_hooks         /* register hooks                      */
};

